#!/usr/bin/perl -w

##    GCertManager v0.1 - (c) Julien Deriviere, 2012 - GPL release
##
##    This program is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with this program.  If not, see <http://www.gnu.org/licenses/>.

my $ca_name="MyCA";

$ENV{'GC_COUNTRY'}='FR';
$ENV{'GC_PROVINCE'}='IDF';
$ENV{'GC_LOCALITY'}='Paris';
$ENV{'GC_ORGANIZATION'}='MyCompany';
$ENV{'GC_ORGUNIT'}='Adminz';
$ENV{'GC_COMMON_NAME'}=$ca_name;
$ENV{'GC_EMAIL'}='admin@mycompany.com';

###################################################
#### DO NOT TOUCH ANYTHING BELOW THIS LINE!!!! ####
###################################################


use strict;
use POSIX;
use Term::Cap;
use Term::ANSIColor;
use Cwd 'abs_path';
use File::Basename;


my $my_path=abs_path($0);
my ($_tmp, $my_dir)=fileparse($my_path);
$my_dir=~s/\/$//so;
my $openssl_root=$my_dir;

$ENV{'PATH'}="/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin";
my $openssl_version=`openssl version`; chomp $openssl_version;

my $COL_RED="\x1b\x5b1;31;40m";
my $COL_GREEN="\x1b\x5b1;32;40m";
my $COL_YELLOW="\x1b\x5b1;33;40m";
my $COL_BLUE="\x1b\x5b1;34;40m";
my $COL_PURPLE="\x1b\x5b1;35;40m";
my $COL_TURQUOISE="\x1b\x5b1;36;40m";
my $COL_NORMAL="\x1b\x5b0;37;40m";
my $COL_WHITE="\x1b\x5b0;38;40m";

my $bk_GC_COUNTRY;
my $bk_GC_PROVINCE;
my $bk_GC_LOCALITY;
my $bk_GC_ORGANIZATION;
my $bk_GC_ORGUNIT;
my $bk_GC_COMMON_NAME;
my $bk_GC_EMAIL;

$|=1;
my $terminal=Term::Cap->Tgetent ({OSPEED=>9600});

sub clearScreen{print $terminal->Tputs('cl');}
sub clearHeader
{
        clearScreen();
        print $COL_YELLOW."  -= GCertManager v0.1 - (c) Julien Deriviere, 2012 - contact\@gradew.net =-\n".$COL_NORMAL;
        print $COL_YELLOW."                  Using $openssl_version\n".$COL_NORMAL;
        print "\n";
}

sub subMenuFooter
{
        print "\n";
        print $COL_WHITE."Press ENTER to return to the main menu.".$COL_NORMAL;
        my $input=<STDIN>;
}

sub pushEnv
{
        $bk_GC_COUNTRY=$ENV{'GC_COUNTRY'};
        $bk_GC_PROVINCE=$ENV{'GC_PROVINCE'};
        $bk_GC_LOCALITY=$ENV{'GC_LOCALITY'};
        $bk_GC_ORGANIZATION=$ENV{'GC_ORGANIZATION'};
        $bk_GC_ORGUNIT=$ENV{'GC_ORGUNIT'};
        $bk_GC_COMMON_NAME=$ENV{'GC_COMMON_NAME'};
        $bk_GC_EMAIL=$ENV{'GC_EMAIL'};
}

sub popEnv
{
        $ENV{'GC_COUNTRY'}=$bk_GC_COUNTRY;
        $ENV{'GC_PROVINCE'}=$bk_GC_PROVINCE;
        $ENV{'GC_LOCALITY'}=$bk_GC_LOCALITY;
        $ENV{'GC_ORGANIZATION'}=$bk_GC_ORGANIZATION;
        $ENV{'GC_ORGUNIT'}=$bk_GC_ORGUNIT;
        $ENV{'GC_COMMON_NAME'}=$bk_GC_COMMON_NAME;
        $ENV{'GC_EMAIL'}=$bk_GC_EMAIL;
}

sub genCA
{
        clearHeader();
        $ENV{'GC_ROOT_STRUCT'}=$openssl_root;
        $ENV{'OPENSSL_CONF'}="$openssl_root/openssl.conf";
        $ENV{'GC_ROOT_CA'}="$openssl_root/ca";
        $ENV{'GC_DEFAULT_DAYS'}="3650";
        $ENV{'GC_COMMON_NAME'}=$ca_name;

        print "\n";
        print $COL_RED."!! WARNING !!\n";
        print "This will erase everything in ".$COL_PURPLE."$openssl_root/ca\n".$COL_NORMAL;
        print "Are you sure (y/n) ?";
        my $sure=<STDIN>; chomp $sure;
        if($sure!~/^\s*y(es)?\s*$/soi)
        {
                return;
        }

        system("rm -rf $openssl_root/ca");
        system("mkdir -p $openssl_root ; mkdir $openssl_root/ca ; mkdir $openssl_root/ca/private ; mkdir $openssl_root/ca/certs ; mkdir $openssl_root/ca/req ; chmod o-rxw $openssl_root/ca");
        system("echo '01' > $openssl_root/ca/serial");
        system("echo '01' > $openssl_root/ca/crlnumber");
        system("touch $openssl_root/ca/index.txt");
        print "Repository rebuilt.\n";

        my $ret=system("openssl req -batch -new -x509 -extensions CA -keyout $openssl_root/ca/private/cakey.pem -out $openssl_root/ca/cacert.pem -days 3650");
        print "\n";
        if($ret==0){
                print $COL_GREEN."CA created successfully!\n".$COL_NORMAL;
        }else{
                print $COL_RED."CA creation failed!\n".$COL_NORMAL;
        }
        subMenuFooter();
}

sub seeAllReq
{
        my @tab_req=<$openssl_root/ca/req/*.req>;
        my @tab_fin;
        foreach my $file(@tab_req)
        {
                $file=~m/^.*\/([^\/]+)\.req/so;
                push @tab_fin, $1;
        }
        if(scalar @tab_fin>0)
        {
                print "Pending requests:\n";
                print join ", ",@tab_fin;
                print "\n";
        }else{
                print "There are no pending requests.\n";
                return 0;
        }
        print "\n";
        return scalar @tab_fin;
}

sub seeAllKnownCerts
{
        my @tab_req=<$openssl_root/ca/certs/*.pem>;
        my @tab_fin;
        foreach my $file(@tab_req)
        {
                $file=~m/^.*\/([^\/]+)\.pem/so;
                $file=$1;
                if($file!~/^[0-9]+$/so)
                {
                        push @tab_fin, $1;
                }
        }
        if(scalar @tab_fin>0)
        {
                print "Certificates:\n";
                print join ", ",@tab_fin;
                print "\n\n";
        }else{
                print "There are no signed certificates.\n";
                print "\n";
                return 0;
        }
        return scalar @tab_fin;
}

sub requestCertClient
{
        clearHeader();
        pushEnv();
        my ($country, $province, $locality, $organization, $orgunit, $commonName, $email);
        $ENV{'GC_COMMON_NAME'}='client_name';
        printf("Country (%s) :",$bk_GC_COUNTRY); $country=<STDIN>; chomp $country; $country=~s/^\s*//so; $country=~s/\s*$//so; if($country eq ""){$country=$bk_GC_COUNTRY;};
        printf("Province (%s) :",$bk_GC_PROVINCE); $province=<STDIN>; chomp $province; $province=~s/^\s*//so; $province=~s/\s*$//so; if($province eq ""){$province=$bk_GC_PROVINCE;};
        printf("Locality (%s) :",$bk_GC_LOCALITY); $locality=<STDIN>; chomp $locality; $locality=~s/^\s*//so; $locality=~s/\s*$//so; if($locality eq ""){$locality=$bk_GC_LOCALITY;};
        printf("Organization (%s) :",$bk_GC_ORGANIZATION); $organization=<STDIN>; chomp $organization; $organization=~s/^\s*//so; $organization=~s/\s*$//so; if($organization eq ""){$organization=$bk_GC_ORGANIZATION;};
        printf("Org. unit (%s) :",$bk_GC_ORGUNIT); $orgunit=<STDIN>; chomp $orgunit; $orgunit=~s/^\s*//so; $orgunit=~s/\s*$//so; if($orgunit eq ""){$orgunit=$bk_GC_ORGUNIT;};
        printf("Common Name (%s) :",$ENV{'GC_COMMON_NAME'}); $commonName=<STDIN>; chomp $commonName; $commonName=~s/^\s*//so; $commonName=~s/\s*$//so; if($commonName eq ""){$commonName=$ENV{'GC_COMMON_NAME'};};
        printf("E-mail (%s) :",$bk_GC_EMAIL); $email=<STDIN>; chomp $email; $email=~s/^\s*//so; $email=~s/\s*$//so; if($email eq ""){$email=$bk_GC_EMAIL;};
        $ENV{'GC_COUNTRY'}=$country;
        $ENV{'GC_PROVINCE'}=$province;
        $ENV{'GC_LOCALITY'}=$locality;
        $ENV{'GC_ORGANIZATION'}=$organization;
        $ENV{'GC_ORGUNIT'}=$orgunit;
        $ENV{'GC_COMMON_NAME'}=$commonName;
        $ENV{'GC_EMAIL'}=$email;

        $ENV{'OPENSSL_CONF'}="$openssl_root/openssl.conf";
        $ENV{'GC_ROOT_CA'}="$openssl_root/ca";
        $ENV{'GC_DEFAULT_DAYS'}="365";
        my $ret=system("openssl req -batch -new -nodes -out $openssl_root/ca/req/$commonName.req -keyout $openssl_root/ca/req/$commonName.key");
        print "\n";
        if($ret==0){
                print $COL_GREEN."Certificate request successful!\n".$COL_NORMAL;
        }else{
                print $COL_RED."Certificate request failed!\n".$COL_NORMAL;
        }

        popEnv();
        subMenuFooter();
}

sub requestCertServer
{
        clearHeader();
        pushEnv();
        my ($country, $province, $locality, $organization, $orgunit, $commonName, $email);
        $ENV{'GC_COMMON_NAME'}='server_name';
        printf("Country (%s) :",$bk_GC_COUNTRY); $country=<STDIN>; chomp $country; $country=~s/^\s*//so; $country=~s/\s*$//so; if($country eq ""){$country=$bk_GC_COUNTRY;};
        printf("Province (%s) :",$bk_GC_PROVINCE); $province=<STDIN>; chomp $province; $province=~s/^\s*//so; $province=~s/\s*$//so; if($province eq ""){$province=$bk_GC_PROVINCE;};
        printf("Locality (%s) :",$bk_GC_LOCALITY); $locality=<STDIN>; chomp $locality; $locality=~s/^\s*//so; $locality=~s/\s*$//so; if($locality eq ""){$locality=$bk_GC_LOCALITY;};
        printf("Organization (%s) :",$bk_GC_ORGANIZATION); $organization=<STDIN>; chomp $organization; $organization=~s/^\s*//so; $organization=~s/\s*$//so; if($organization eq ""){$organization=$bk_GC_ORGANIZATION;};
        printf("Org. unit (%s) :",$bk_GC_ORGUNIT); $orgunit=<STDIN>; chomp $orgunit; $orgunit=~s/^\s*//so; $orgunit=~s/\s*$//so; if($orgunit eq ""){$orgunit=$bk_GC_ORGUNIT;};
        printf("Common Name (%s) :",$ENV{'GC_COMMON_NAME'}); $commonName=<STDIN>; chomp $commonName; $commonName=~s/^\s*//so; $commonName=~s/\s*$//so; if($commonName eq ""){$commonName=$ENV{'GC_COMMON_NAME'};};
        printf("E-mail (%s) :",$bk_GC_EMAIL); $email=<STDIN>; chomp $email; $email=~s/^\s*//so; $email=~s/\s*$//so; if($email eq ""){$email=$bk_GC_EMAIL;};
        $ENV{'GC_COUNTRY'}=$country;
        $ENV{'GC_PROVINCE'}=$province;
        $ENV{'GC_LOCALITY'}=$locality;
        $ENV{'GC_ORGANIZATION'}=$organization;
        $ENV{'GC_ORGUNIT'}=$orgunit;
        $ENV{'GC_COMMON_NAME'}=$commonName;
        $ENV{'GC_EMAIL'}=$email;

        $ENV{'OPENSSL_CONF'}="$openssl_root/openssl.conf";
        $ENV{'GC_ROOT_CA'}="$openssl_root/ca";
        $ENV{'GC_DEFAULT_DAYS'}="365";
        my $ret=system("openssl req -batch -new -extensions SERVER -nodes -out $openssl_root/ca/req/$commonName.req -keyout $openssl_root/ca/req/$commonName.key");
        print "\n";
        if($ret==0){
                print $COL_GREEN."Certificate request successful!\n".$COL_NORMAL;
        }else{
                print $COL_RED."Certificate request failed!\n".$COL_NORMAL;
        }

        popEnv();
        subMenuFooter();
}

sub signCertClient
{
        clearHeader();
        seeAllReq();
        print "Enter the name of the certificate you wish to sign :";
        my $commonName=<STDIN>; chomp $commonName;
        $commonName=~s/^\s*//so;$commonName=~s/\s*$//so;
        if($commonName eq ""){return;};

        ##
        my ($s_c, $s_st, $s_l, $s_o, $s_ou, $s_cn, $s_email)=("","","","","","","");
        my $subject=`openssl req -noout -in $openssl_root/ca/req/$commonName.req -subject`;
        chomp $subject;
        if($subject=~/^subject=(.+)$/so)
        {
                $subject=$1;
                if($subject=~/C=([^\/]+)/so){$s_c=$1;};
                if($subject=~/ST=([^\/]+)/so){$s_st=$1;};
                if($subject=~/L=([^\/]+)/so){$s_l=$1;};
                if($subject=~/O=([^\/]+)/so){$s_o=$1;};
                if($subject=~/OU=([^\/]+)/so){$s_ou=$1;};
                if($subject=~/CN=([^\/]+)/so){$s_cn=$1;};
                if($subject=~/emailAddress=([^\/]+)/so){$s_email=$1;};
        }else{
                print $COL_RED."ERROR: Could not read subject information from CSR file!\n".$COL_NORMAL;
                subMenuFooter();
                return;
        }
        printf("Country      : %s\n", $s_c);
        printf("Province     : %s\n", $s_st);
        printf("Locality     : %s\n", $s_l);
        printf("Organization : %s\n", $s_o);
        printf("Org. unit    : %s\n", $s_ou);
        printf("Common name  : %s\n", $s_cn);
        printf("E-mail       : %s\n", $s_email);
        print "\n";

        print "Enter a validity period (in days)                  :";
        my $days=<STDIN>; chomp $days;
        $days=~s/^\s*//so;$days=~s/\s*$//so;
        if($days!~/^[0-9]+$/so){return;};

        $ENV{'OPENSSL_CONF'}="$openssl_root/openssl.conf";
        $ENV{'GC_ROOT_CA'}="$openssl_root/ca";
        $ENV{'GC_DEFAULT_DAYS'}=$days;
        $ENV{'GC_COMMON_NAME'}=$commonName;
        my $ret=system("openssl ca -batch -out $openssl_root/ca/certs/$commonName.pem -infiles $openssl_root/ca/req/$commonName.req");
        print "\n";
        if($ret==0){
                print $COL_GREEN."Certificate signature successful!\n".$COL_NORMAL;
                unlink("$openssl_root/ca/req/$commonName.req");
        }else{
                print $COL_RED."Certificate signature failed!\n".$COL_NORMAL;
        }

        subMenuFooter();
}

sub signCertServer
{
        clearHeader();
        seeAllReq();
        print "Enter the name of the certificate you wish to sign :";
        my $commonName=<STDIN>; chomp $commonName;
        $commonName=~s/^\s*//so;$commonName=~s/\s*$//so;
        if($commonName eq ""){return;};

        my ($s_c, $s_st, $s_l, $s_o, $s_ou, $s_cn, $s_email)=("","","","","","","");
        my $subject=`openssl req -noout -in $openssl_root/ca/req/$commonName.req -subject`;
        chomp $subject;
        if($subject=~/^subject=(.+)$/so)
        {
                $subject=$1;
                if($subject=~/C=([^\/]+)/so){$s_c=$1;};
                if($subject=~/ST=([^\/]+)/so){$s_st=$1;};
                if($subject=~/L=([^\/]+)/so){$s_l=$1;};
                if($subject=~/O=([^\/]+)/so){$s_o=$1;};
                if($subject=~/OU=([^\/]+)/so){$s_ou=$1;};
                if($subject=~/CN=([^\/]+)/so){$s_cn=$1;};
                if($subject=~/emailAddress=([^\/]+)/so){$s_email=$1;};
        }else{
                print $COL_RED."ERROR: Could not read subject information from CSR file!\n".$COL_NORMAL;
                subMenuFooter();
                return;
        }
        printf("Country      : %s\n", $s_c);
        printf("Province     : %s\n", $s_st);
        printf("Locality     : %s\n", $s_l);
        printf("Organization : %s\n", $s_o);
        printf("Org. unit    : %s\n", $s_ou);
        printf("Common name  : %s\n", $s_cn);
        printf("E-mail       : %s\n", $s_email);
        print "\n";

        print "Enter a validity period (in days)                  :";
        my $days=<STDIN>; chomp $days;
        $days=~s/^\s*//so;$days=~s/\s*$//so;
        if($days!~/^[0-9]+$/so){return;};

        $ENV{'OPENSSL_CONF'}="$openssl_root/openssl.conf";
        $ENV{'GC_ROOT_CA'}="$openssl_root/ca";
        $ENV{'GC_DEFAULT_DAYS'}=$days;
        $ENV{'GC_COMMON_NAME'}=$commonName;
        my $ret=system("openssl ca -batch -extensions SERVER -out $openssl_root/ca/certs/$commonName.pem -infiles $openssl_root/ca/req/$commonName.req");
        print "\n";
        if($ret==0){
                print $COL_GREEN."Certificate signature successful!\n".$COL_NORMAL;
                unlink("$openssl_root/ca/req/$commonName.req");
        }else{
                print $COL_RED."Certificate signature failed!\n".$COL_NORMAL;
        }

        subMenuFooter();
}

sub genCRL
{
        clearHeader();
        $ENV{'OPENSSL_CONF'}="$openssl_root/openssl.conf";
        $ENV{'GC_ROOT_CA'}="$openssl_root/ca";
        $ENV{'GC_DEFAULT_DAYS'}=365;
        $ENV{'GC_COMMON_NAME'}=$ca_name;
        my $ret=system("openssl ca -gencrl -out $openssl_root/ca/crl.pem");
        print "\n";
        if($ret==0) {
                print $COL_GREEN."CRL generated successfully!\n".$COL_NORMAL;
        }else{
                print $COL_RED."CRL generation failed!\n".$COL_NORMAL;
        }
        subMenuFooter();
}

sub revokeCert
{
        clearHeader();
        if(seeAllKnownCerts()==0)
        {
                subMenuFooter();
                return;
        }
        print "Enter the name of the certificate you wish to revoke :";
        my $commonName=<STDIN>; chomp $commonName;
        $commonName=~s/^\s*//so;$commonName=~s/\s*$//so;
        if($commonName eq ""){return;};

        $ENV{'OPENSSL_CONF'}="$openssl_root/openssl.conf";
        $ENV{'GC_ROOT_CA'}="$openssl_root/ca";
        $ENV{'GC_DEFAULT_DAYS'}="365";
        $ENV{'GC_COMMON_NAME'}=$ca_name;

        my $ret=system("openssl ca -revoke $openssl_root/ca/certs/$commonName.pem -crl_reason keyCompromise");
        print "\n";
        if($ret==0) {
                print $COL_GREEN."Certificate revocation successful!\n".$COL_NORMAL;
        }else{
                print $COL_RED."Certificate revocation failed!\n".$COL_NORMAL;
        }

        subMenuFooter();
}

sub seeAllCerts
{
        clearHeader();
        print "List of all signed/revoked certificates:\n";
# V       130203145147Z           01      unknown /C=FR/ST=IDF/O=Gradew/OU=Administration/CN=server.gradew.net/emailAddress=admin@gradew.net
        open LIST,"<$openssl_root/ca/index.txt";
        my $line;
        my $itemsFound=0;
        while($line=<LIST>)
        {
                chomp $line;
                my @tab_line=split /[\t\s ]+/, $line;
                my $status=$tab_line[0];
                my ($created, $serial, $dn);
                if($status eq "V")
                {
                        $created=$tab_line[1];
                        $serial=$tab_line[2];
                        $dn=$tab_line[4];
                        for(my $j=5;$j<(scalar @tab_line);$j++)
                        {
                                $dn.=" ".$tab_line[$j];
                        }
                }else{
                        $created=$tab_line[1];
                        $serial=$tab_line[3];
                        $dn=$tab_line[5];
                        for(my $j=6;$j<(scalar @tab_line);$j++)
                        {
                                $dn.=" ".$tab_line[$j];
                        }

                }
                $dn=~m/CN=([^\/]+)/; my $cn=$1;
                $dn=~m/emailAddress=([^\/]+)/; my $email=$1;
                if($status eq "V") {
                        $status="VALID";
                }elsif($status eq "R") {
                        $status="REVOKED";
                }elsif($status eq "E") {
                        $status="EXPIRED";
                }else{
                        $status="UNKNOWN (".$status.")";
                }
                print "$status => $cn, SERIAL=$serial, EMAIL=$email\n";
                $itemsFound++;
        }
        close LIST;
        if($itemsFound==0)
        {
                print "No entries found.\n";
        }
        subMenuFooter();
}

sub seePendingReq
{
        clearHeader();
        seeAllReq();
        subMenuFooter();
}

sub sanitizeDirectory
{
        my $input=$_[0];
        chomp $input;
        $input=~s/^\s*//so;
        $input=~s/\s*$//so;
        $input=~s/\/+/\//so;
        if($input!~/^\//so)
        {
                return "";
        }
        return $input;
}

sub changeDir
{
        clearHeader();
        print "Current directory :$openssl_root\n";
        my $finished=0;
        while(!$finished)
        {
                print "New directory     :";
                my $input=<STDIN>; chomp $input;
                if($input=~/^\s*$/so)
                {
                        $finished++;
                        last;
                }
                my $sanitized=sanitizeDirectory($input);
                if($sanitized eq "")
                {
                        print "REFUSED: Path is incorrect - must be absolute !!\n";
                        next;
                }
                $openssl_root=$sanitized;
                $finished++;
        }
}

sub configure
{
        clearHeader();
        my ($country, $province, $locality, $organization, $orgunit, $commonName, $email);
        printf("Country (%s) :", $ENV{'GC_COUNTRY'}); $country=<STDIN>; chomp $country;
        printf("Province (%s) :", $ENV{'GC_PROVINCE'}); $province=<STDIN>; chomp $province;
        printf("Locality (%s) :", $ENV{'GC_LOCALITY'}); $locality=<STDIN>; chomp $locality;
        printf("Organization (%s) :", $ENV{'GC_ORGANIZATION'}); $organization=<STDIN>; chomp $organization;
        printf("Org. unit (%s) :", $ENV{'GC_ORGUNIT'}); $orgunit=<STDIN>; chomp $orgunit;
        printf("Common Name (%s) :", $ENV{'GC_COMMON_NAME'}); $commonName=<STDIN>; chomp $commonName;
        printf("E-mail (%s) :", $ENV{'GC_EMAIL'}); $email=<STDIN>; chomp $email;

        if($country!~/^\s*$/so){$ENV{'GC_COUNTRY'}=$country;};
        if($province!~/^\s*$/so){$ENV{'GC_PROVINCE'}=$province;};
        if($locality!~/^\s*$/so){$ENV{'GC_LOCALITY'}=$locality;};
        if($organization!~/^\s*$/so){$ENV{'GC_ORGANIZATION'}=$organization;};
        if($orgunit!~/^\s*$/so){$ENV{'GC_ORGUNIT'}=$orgunit;};
        if($commonName!~/^\s*$/so){$ENV{'GC_COMMON_NAME'}=$commonName;};
        if($email!~/^\s*$/so){$ENV{'GC_EMAIL'}=$email;};

        subMenuFooter();
}

sub buildPackage
{
        clearScreen();
        seeAllKnownCerts();
        my $finished=0;
        my $input;
        my $fName;
        while(!$finished)
        {
                print "Certificate to package :";
                $input=<STDIN>; chomp $input;
                if($input=~/^\s*$/so)
                {
                        $finished++;
                        return;
                }
                $fName="$openssl_root/ca/certs/".$input.".pem";
                if(!(-e $fName))
                {
                        print "This certificate does not exist!\n";
                        next;
                }
                $finished++;
        }
        print "Building package for $input...\n";
        my $dir="./certif_$input";
        my $ret=system("cd $openssl_root ; mkdir $dir ; cp ca/cacert.pem ca/crl.pem ca/req/$input.key ca/certs/$input.pem $dir/ ; tar cjvf certif_$input.tbz2 $dir/ ; rm -rf $dir ; cd - >/dev/null 2>&1");
        print "\n";
        if($ret==0) {
                print $COL_GREEN."Package built!\n".$COL_NORMAL;
        }else{
                print $COL_RED."Could not build package!\n".$COL_NORMAL;
        }
        subMenuFooter();
}

sub mainMenu
{
        my $running=1;
        my $choice;
        my $sanitized=sanitizeDirectory($openssl_root);
        if($sanitized eq "")
        {
                changeDir();
        }
        while($running)
        {
                clearHeader();
                print "  1. Create Certificate Authority (will erase all existing data!)\n";
                print "  2. Request server certificate\n";
                print "  3. Request client certificate\n";
                print "  4. Sign server certificate\n";
                print "  5. Sign client certificate\n";
                print "  6. Generate CRL\n";
                print "\n";
                print "  7. See all signed/revoked certificates\n";
                print "  8. See all pending requests\n";
                print "\n";
                print "  9. Revoke certificate\n";
                print "\n";
                print "  10. Change directory\n";
                print $COL_TURQUOISE."      ==>  ".$openssl_root."\n".$COL_NORMAL;
                printf("  11. Configure identity\n");
                printf($COL_TURQUOISE."      ==>  C=%s, ST=%s, L=%s\n", $ENV{'GC_COUNTRY'}, $ENV{'GC_PROVINCE'}, $ENV{'GC_LOCALITY'});
                printf("      ==>  O=%s, OU=%s\n", $ENV{'GC_ORGANIZATION'}, $ENV{'GC_ORGUNIT'});
                printf("      ==>  CN=%s, EMAIL=%s\n$COL_NORMAL", $ENV{'GC_COMMON_NAME'}, $ENV{'GC_EMAIL'});
                print "  12. Build a package\n";
                print "\n";
                print "Your choice (or nothing to quit): ";
                $choice=<STDIN>; chomp $choice;
                if($choice=~/^\s*$/so)
                {
                        $choice=0;
                        $running=0;
                        last;
                }
                if($choice=~/^\s*([0-9]+)\s*$/so)
                {
                        $choice=$1;
                }else{
                        next;
                }

                if($choice==0) {
                        $running=0;
                        last;
                }
                if($choice==1) {
                        genCA();
                }elsif($choice==2) {
                        requestCertServer();
                }elsif($choice==3) {
                        requestCertClient();
                }elsif($choice==4) {
                        signCertServer();
                }elsif($choice==5) {
                        signCertClient();
                }elsif($choice==6) {
                        genCRL();
                }elsif($choice==7) {
                        seeAllCerts();
                }elsif($choice==8) {
                        seePendingReq();
                }elsif($choice==9) {
                        revokeCert();
                }elsif($choice==10) {
                        changeDir();
                }elsif($choice==11) {
                        configure();
                }elsif($choice==12) {
                        buildPackage();
                }
        }
}

##############################

mainMenu();
print "\n";
print "    GCertManager  Copyright (C) 2012  Julien Deriviere\n";
print "    This program comes with ABSOLUTELY NO WARRANTY; for details type 'show w'.\n";
print "    This is free software, and you are welcome to redistribute it\n";
print "    under certain conditions; type `show c' for details.\n";
print "\n";
print "Bubye!\n";
